import { _RouteRecordBase } from 'vue-router';
import { RawRouteComponent, NavigationGuardReturn } from '../vue-router-utils';
import { RoutiderPath, RoutiderPaths, ExtractParams } from '../options/path';
import { RoutiderLocation, RoutiderLocationN } from './location';
import { RoutiderNavigationGuardNext } from '../router/navigationGuard';
/**
 * Typed _RouteRecordProps
 */
export declare type _RouteRecordProps<Params extends string | undefined> = boolean | Record<string, any> | ((to: RoutiderLocation<Params, undefined>) => Record<string, any>);
declare type RoutiderRouteRecordRedirectOption<Params extends string | undefined> = (to: RoutiderLocation<Params, undefined>) => unknown;
/**
 * Typed `NavigationGuardWithThis` for beforeEnter
 */
export interface RoutiderBeforeEnterGuardWithThis<T, Params extends string | undefined> {
    (this: T, to: RoutiderLocationN<Params, undefined>, from: RoutiderLocationN<undefined, undefined>, next: RoutiderNavigationGuardNext<Record<never, never>>): NavigationGuardReturn | Promise<NavigationGuardReturn>;
}
/**
 * Typed RouteRecordBase
 */
export interface _RoutiderRouteRecordBase<T extends string | undefined> extends Omit<_RouteRecordBase, 'path' | 'beforeEnter'> {
    /**
     * `name` will be automatically set from key
     */
    name?: never;
    /**
     * Path of the record. Should start with / unless the record is the child of another record.
     * @example createPath`/users/${'id'}` matches `/users/1` as well as `/users/posva`. '/users' matches only `/users`.
     */
    path: RoutiderPath<T> | string | RoutiderPaths<T> | string[];
    /**
     * Use `path` with an array for aliases.
     */
    alias?: never;
    /**
     * Nested routes are not supported by vue-routider because of type inference limitations.
     * @see {@link https://github.com/sapphi-red/vue-routider/issues/4}
     */
    children?: never;
    redirect?: RoutiderRouteRecordRedirectOption<T>;
    beforeEnter?: RoutiderBeforeEnterGuardWithThis<undefined, T> | RoutiderBeforeEnterGuardWithThis<undefined, T>[];
}
interface RoutiderRouteRecordSingleView<T extends string | undefined> extends _RoutiderRouteRecordBase<T> {
    /**
     * Component to display when the URL matches this route.
     */
    component: RawRouteComponent;
    /**
     * Allow passing down params as props to the component rendered by `router-view`.
     */
    props?: _RouteRecordProps<T>;
}
interface RoutiderRouteRecordMultipleViews<T extends string | undefined> extends _RoutiderRouteRecordBase<T> {
    /**
     * Components to display when the URL matches this route. Allow using named views.
     */
    components: Record<string, RawRouteComponent>;
    /**
     * Allow passing down params as props to the component rendered by
     * `router-view`. Should be an object with the same keys as `components` or a
     * boolean to be applied to every component.
     */
    props?: Record<string, _RouteRecordProps<T>> | boolean;
}
interface RoutiderRouteRecordRedirect<T extends string | undefined> extends _RoutiderRouteRecordBase<T> {
    redirect: RoutiderRouteRecordRedirectOption<T>;
    component?: never;
    components?: never;
    children?: never;
}
export declare type RoutiderRouteRecord<T extends string | undefined = string | undefined> = RoutiderRouteRecordSingleView<T> | RoutiderRouteRecordMultipleViews<T> | RoutiderRouteRecordRedirect<T>;
interface PathAndAlias {
    path: string;
    alias?: string[];
}
export declare const pathToPathAndAlias: (path: RoutiderPath | string | RoutiderPaths | string[]) => PathAndAlias;
declare type ConvertToUndefinedIfStringPath<Route extends RoutiderRouteRecord> = string extends ExtractParams<Route['path']> ? RoutiderRouteRecord<undefined> : Route;
export declare const createRoute: <T extends string | undefined>(route: RoutiderRouteRecord<T>) => ConvertToUndefinedIfStringPath<RoutiderRouteRecord<T>>;
export {};
